@@ -18,31 +18,34 @@ package EBox::Backup;
 use strict;
 use warnings;
 
-use File::Temp qw(tempdir);
-use File::Copy qw(copy move);
-use File::Slurp qw(read_file write_file);
-use File::Basename;
 use EBox::Config;
 use EBox::Global;
 use EBox::Exceptions::Internal;
 use EBox::Exceptions::MissingArgument;
 use EBox::Gettext;
 use EBox::FileSystem;
+use EBox::ProgressIndicator;
+use EBox::ProgressIndicator::Dummy;
+
+use File::Temp qw(tempdir);
+use File::Copy qw(copy move);
+use File::Slurp qw(read_file write_file);
+use File::Basename;
+
 use Error qw(:try);
 use Digest::MD5;
 use EBox::Sudo qw(:all);
 use POSIX qw(strftime);
 use DirHandle;
 use Perl6::Junction qw(any all);
-use EBox::Backup::FileBurner;
-use EBox::Backup::OpticalDiscDrives;
-use Params::Validate qw(validate_with validate_pos);
+
+use Params::Validate qw(validate_with validate_pos ARRAYREF);
+use Filesys::Df;
 
 use Readonly;
 Readonly::Scalar our $FULL_BACKUP_ID  => 'full backup';
 Readonly::Scalar our $CONFIGURATION_BACKUP_ID  =>'configuration backup';
 Readonly::Scalar our $BUGREPORT_BACKUP_ID  =>'bugreport configuration dump';
-Readonly::Scalar my $DISC_BACKUP_FILE  => 'eboxbackup.tar';
 my $RECURSIVE_DEPENDENCY_THRESHOLD = 3;
 
 sub new 
@@ -59,9 +62,11 @@ sub _makeBackup # (description, bug?)
 {
 	my ($self, %options) = @_;
 
-	my $bug         = delete $options{bug};
 	my $description = delete $options{description};
 
+	my $bug         = $options{bug};
+	my $progress   = $options{progress};
+
 	my $time = strftime("%F %T", localtime);
 
 	my $confdir = EBox::Config::conf;
@@ -85,6 +90,13 @@ sub _makeBackup # (description, bug?)
 	  }
 
 
+	  if ($progress) {
+	    $progress->setMessage(__('Creating backup archive'));
+	    $progress->notifyTick();
+	  }
+
+
+
 	  my $filesArchive  = "$archiveContentsDir/files.tgz";
 	  $self->_createFilesArchive($auxDir, $filesArchive);
 	  $self->_createMd5DigestForArchive($filesArchive, $archiveContentsDir);
@@ -112,15 +124,30 @@ sub _makeBackup # (description, bug?)
 
 sub _dumpModulesBackupData
 {
-  my ($self, $auxDir, @options) = @_;
+  my ($self, $auxDir, %options) = @_;
+
+  my $progress   = $options{progress};
 
   my $global = EBox::Global->getInstance();
   my @names = @{$global->modNames};
-  foreach (@names) {
-    my $mod = $global->modInstance($_);
+  foreach my $modName (@names) {
+    # XXX temporally skipping logs
+    next if $modName eq 'logs';
+
+    my $mod = $global->modInstance($modName);
+
+
+    if ($progress) {
+      # update progress object
+      $progress->notifyTick();
+      $progress->setMessage(__x('Dumping configuration of module {m}',
+			      m => $modName));
+    }
+
+
     try {
-      EBox::debug("Dumping $_ backup data");
-      $mod->makeBackup($auxDir, @options);
+      EBox::debug("Dumping $modName backup data");
+      $mod->makeBackup($auxDir, %options);
     } 
     catch EBox::Exceptions::Base with {
       my $ex = shift;
@@ -317,47 +344,11 @@ sub backupDetails # (id)
   return $details;
 }
 
-#
-# Method: backupDiscDetails
-#
-#      Gathers the details from a backup stored in a CD-ROM or DVD-ROM.
-#
-# Limitations:
-#       It is assumed that is only a backup file per disc. If there are multiple disc with backups wich one is selected is not defined
-#
-# Parameters:
-#
-#       
-#
-# Returns:
-#
-#       A hash reference with the details. This hash consists of:
-#	
-#       file - the filename of the archive
-#	date - when it was backed up
-#	description - backup's description
-#       disc - boolean that marks whether the file is into a optical disc or not
-#   	
-#
-sub backupDiscDetails
-{
-  my ($self) = @_;
-
-  my $discFileInfo = EBox::Backup::OpticalDiscDrives::searchFileInDiscs($DISC_BACKUP_FILE);
-  if (!defined $discFileInfo) {
-    throw EBox::Exceptions::External(__('Insert a backup disc and try again, please'));
-  }
-
-  my $details = $self->backupDetailsFromArchive($discFileInfo->{file});
-  $details->{disc} = 1;
-  
-  return $details;
-}
 
 #
 # Method: backupDetailsFromArchive
 #
-#      Gathers the details of the bckup stored in a given file
+#      Gathers the details of the backup stored in a given file
 #
 #
 # Parameters:
@@ -538,6 +529,54 @@ sub _ensureBackupdirExistence
 } 
 
 
+
+
+#
+# Method: prepareMakeBackup
+#
+#   	Prepares a backup restauration 
+#
+# Parameters:
+#
+#                   description - backup's description (backwards compability mode)
+#                   fullBackup  - wether do a full backup or  backup only configuration (default: false)
+#
+#  Returns:
+#    the progress indicator object whihc represents the progress of the restauration
+#
+# Exceptions:
+#	
+#	External - If it can't unpack de backup
+#
+sub prepareMakeBackup
+{
+  my ($self, %options) = @_;
+
+  # make sure description is scaped
+  $options{description} = q{'} . $options{description} . q{'};
+  my @scriptParams = %options; 
+
+  my $makeBackupScript = EBox::Config::pkgdata() . 'ebox-make-backup';
+  $makeBackupScript    .= "  @scriptParams";
+  
+  my $global     = EBox::Global->getInstance();
+  my $totalTicks = scalar @{ $global->modNames } + 2; # there are one task for
+                                                      # each module plus two
+                                                      # tasks for writing the
+                                                      # archive  file
+           
+
+  my $progressIndicator =  EBox::ProgressIndicator->create(
+			     executable => $makeBackupScript,
+			     totalTicks => $totalTicks,
+						    );
+
+  $progressIndicator->runExecutable();
+
+  return $progressIndicator;
+}
+
+
 #
 # Method: makeBackup 
 #
@@ -545,9 +584,10 @@ sub _ensureBackupdirExistence
 #
 # Parameters:
 #
-#                   description - backup's description (backwards compability mode)
+#                   progress-  progress indicator 
+#                       associated with this operation (optionak)
+#                   description - backup's description (default: 'Backup')
 #                   fullBackup  - wether do a full backup or  backup only configuration (default: false)
-#                   directlyToDisc      - burn directly to Cd the backup and do not store it in the filesystem (default: false)
 #
 # Exceptions:
 #	
@@ -559,25 +599,60 @@ sub makeBackup # (options)
   validate_with(
 		params => [%options],
 		spec   => {
+			   progress     => { 
+					    optional => 1, 
+					     isa => 'EBox::ProgressIndicator'
+					   },
 			   description => { default =>  __('Backup') },
 			   fullBackup  => { default => 0 },
-			   directlyToDisc  => { default => 0 },
+			   bug         => { default => 0},
 			  });
 
-  my $time = strftime("%F", localtime);
-
-  $self->_modulesReady();
-
-  my $backupdir = backupDir();
-  _ensureBackupdirExistence();
-
-  my $filename = $self->_makeBackup(%options);
-
-  if ($options{directlyToDisc}) {
-    return $self->_moveToDisc($filename);
+  my $progress = $options{progress};
+  if (not $progress) {
+    $progress = EBox::ProgressIndicator::Dummy->create();
+    $options{progress} = $progress;
   }
+
+  EBox::debug("make backup id: " . $progress->id());
+  $progress->started or
+    throw EBox::Exceptions::Internal("ProgressIndicator's executable has not been run");
   
-  return $self->_moveToArchives($filename, $backupdir);
+  
+  my $backupdir = backupDir();
+
+  my $filename;
+  try {
+    $self->_modulesReady();
+    
+    _ensureBackupdirExistence();
+
+    $filename = $self->_makeBackup(%options);
+  }
+ otherwise {
+   my $ex = shift @_;
+   $progress->setAsFinished(1, $ex->text);
+   $ex->throw();
+ };
+
+
+  my $backupFinalPath;
+  try {
+      $progress->notifyTick();
+      $progress->setMessage(__('Writing backup file to hard disc'));
+
+      $backupFinalPath = $self->_moveToArchives($filename, $backupdir);   
+
+      $progress->setAsFinished();   
+  }
+ otherwise {
+   my $ex = shift @_;
+   $progress->setAsFinished(1, $ex->text);
+   $ex->throw();
+ };
+
+
+  return $backupFinalPath;
 }
 
 
@@ -611,14 +686,6 @@ sub  _moveToArchives
   return "$backupdir/$id";
 } 
 
-sub _moveToDisc
-{
-  my ($self, $filename) = @_;
-
-  EBox::Backup::FileBurner::burn(file => $filename);
-  unlink $filename;
-  return undef;
-} 
 
 #
 # Method: makeBugReport
@@ -738,11 +805,14 @@ sub  _checkSize
 
   my $size;
   my $freeSpace;
+  my $safetyFactor = 2; # to be sure we have space left we multiply the backup
+                        # size by this number. The value was guessed, so change
+                        # it if you have better judgment
 
   my $tempDir;
   try {
-    $tempDir = $self->_unpackArchive($archive, 'size');
-    $size = read_file("$tempDir/eboxbackup/size");
+    $tempDir = $self->_unpackArchive($archive, 'size'); 
+    $size = read_file("$tempDir/eboxbackup/size"); # unit -> 1K
   }
   finally {
     if (defined $tempDir) {
@@ -753,47 +823,74 @@ sub  _checkSize
 
 
   my $backupDir = $self->backupDir();
-  my @dfOutput = `/bin/df $backupDir`;
-  my @dfData   = split '\s+', $dfOutput[1];
-  $freeSpace = $dfData[3];
+  $freeSpace = df($backupDir, 1024)->{bfree};
 
-  if ($freeSpace < $size) {
+  if ($freeSpace < ($size*$safetyFactor)) {
     throw EBox::Exceptions::External(__x("There in not enough space left in the hard disk to complete the restore proccess. {size} Kb required. Free sufficient space and retry", size => $size));
   }
 
 } 
 
+ 
 #
-# Method: writeBackupToDisc
+# Method: prepareRestoreBackup
 #
-#    writes the archive to a optical disc. The writer device is chosen automatically. The media is also auto-detected
+#   	Prepares a backup restauration 
 #
 # Parameters:
-# 
-#    id - the id of the archive file
 #
-sub writeBackupToDisc
+#       file - backup's file (as positional parameter) fullRestore - wether do a
+#       full restore or restore only configuration (default: false) 
+#
+#  Returns:
+#    the progress indicator object whihc represents the progress of the restauration
+#
+# Exceptions:
+#	
+#	External - If it can't unpack the backup archive
+#
+sub prepareRestoreBackup
 {
-  my ($self, $id) = @_;
-  validate_pos(@_, 1, 1);
-  
-  $self->_checkId($id);
+  my ($self, $file, %options) = @_;
 
-  my $file = $self->_backupFileById($id);
+  my $restoreBackupScript = EBox::Config::pkgdata() . 'ebox-restore-backup';
 
-  my $backupdir = backupDir();
-  my $destFile = "$backupdir/$DISC_BACKUP_FILE";
-  
-  move($file, $destFile) or throw EBox::Exceptions::Internal("Cannot rename backup file: @!");
-  
-  try {
-      EBox::Backup::FileBurner::burn(file => $destFile);
+  my $execOptions;
+
+  if (exists $options{fullRestore}) {
+    if ($options{fullRestore}) {
+      $execOptions .= '--full-restore ';
+    }
   }
-  finally {
-      move($destFile, $file) or throw EBox::Exceptions::External(__x('Cannot rename backup file from {newName} to his original name {name}. Error message: {error}', newName => $destFile, name => $file, error => $!));
-  };
+
+  if (exists $options{forceDependencies}) {
+    if ($options{forceDependencies}) {
+      $execOptions .= '--force-dependencies ';
+    }
+  }
+
+  if (exists $options{modsToRestore}) {
+    foreach my $m (@{ $options{modsToRestore} }) {
+      $execOptions .= "--module $m ";
+    }
+  }
+
+
+  $restoreBackupScript    .= " $execOptions $file";
+  
+  my $totalTicks = scalar @{ $self->_modInstancesForRestore($file) };
+
+  my $progressIndicator =  EBox::ProgressIndicator->create(
+			     executable => $restoreBackupScript,
+			     totalTicks => $totalTicks,
+						    );
+
+  $progressIndicator->runExecutable();
+
+  return $progressIndicator;
 }
- 
+
+
 #
 # Method: restoreBackup 
 #
@@ -801,8 +898,10 @@ sub writeBackupToDisc
 #
 # Parameters:
 #
-#       file - backup's file (as positional parameter)
-#       fullRestore  - wether do a full restore or  restore only configuration (default: false)
+#       file - backup's file (as positional parameter) 
+#       progressIndicator - Progress indicator associated
+#                       with htis operation (optional )
+# fullRestore - wether do a full restore or restore only configuration (default: false)
 #
 # Exceptions:
 #	
@@ -812,67 +911,157 @@ sub restoreBackup # (file, %options)
 {
   my ($self, $file, %options) = @_;
   defined $file or throw EBox::Exceptions::MissingArgument('Backup file');
+
   validate_with ( params => [%options], 
-		  spec => { fullRestore => { default => 0 },  });
+		  spec => { 
+			   progress    => {
+					   optional => 1,
+					   isa => 'EBox::ProgressIndicator',
+					  },
+			   modsToRestore => {
+					    type => ARRAYREF,
+					    optional => 1,
+					   },
+			   fullRestore => { default => 0 },
+			   dataRestore    => { 
+					   default => 0 ,
+					   # incompatible with fullRestore ..
+					   callbacks => {
+							 incompatibleRestores =>  
+							 sub {
+							   (not $_[0]) or (not $_[1]->{fullRestore})
+							 },
+							},
+					      
+					  },
+			   forceDependencies => {default => 0 },
+			  }
+		);
 
   _ensureBackupdirExistence();
   $self->_checkSize($file);
+
+  my $progress = $options{progress};
+  if (not $progress) {
+    $progress = EBox::ProgressIndicator::Dummy->create;
+    $options{progress} = $progress;
+  }
+
+
+
+  EBox::debug("restore backup id: " . $progress->id);
+  $progress->started or
+    throw EBox::Exceptions::Internal("ProgressIndicator's executable has not been run");
+
   my $tempdir = $self->_unpackAndVerify($file, $options{fullRestore});
 
   try {
-    if (`tar xzf $tempdir/eboxbackup/files.tgz -C $tempdir/eboxbackup`) {
-      `rm -rf $tempdir`;
-      throw EBox::Exceptions::External(
-				       __('Could not unpack the backup'));
-    }
+    $self->_unpackModulesRestoreData($tempdir);
 
-    my @modules  = @{ $self->_modInstancesForRestore() };
+    my @modules  = @{ $self->_modInstancesForRestore($file, %options) };
     my @restored = ();
     try {
       foreach my $mod (@modules) {
-	my $modname = $mod->name();
-	if (-e "$tempdir/eboxbackup/$modname.bak") {
-	  push @restored, $modname;
-	  EBox::debug("Restoring $modname from backup data");
-	  $mod->setAsChanged(); # we set as changed first because it is not guaranteed that
-	  $mod->restoreBackup("$tempdir/eboxbackup", %options);
-	  $self->_migratePackage($mod->package());
+	my $restoreOk = $self->_restoreModule($mod, $tempdir, \%options);
+	if ($restoreOk) {
+	  push @restored, $mod->name();
+	}
 
-	}
-	else {
-	  EBox::error("Restore data not found for module $modname. Skipping $modname restore");
-	}
       }
+
     } 
     otherwise {
       my $ex = shift;
-      EBox::error('Error while restoring: ' . $ex->text());
-      EBox::debug('Error caught: revoking restore for all modules');
-      foreach my $restname (@restored) {
-	my $restmod = EBox::Global->modInstance($restname);
-	$restmod->revokeConfig();
-	# XXX remember no-gconf changes are not revoked!
-	EBox::debug("Revoked changes in $restname module");
-      }
+
+      my $errorMsg = 'Error while restoring: ' . $ex->text();
+      EBox::error($errorMsg);
+      $progress->setAsFinished(1, $errorMsg);
+
+      $self->_revokeRestore(\@restored);
 
       throw $ex;
     };
 
-
     EBox::info('Restore successful');
+    $progress->setAsFinished(); 
   }
   finally {
-      `rm -rf $tempdir`;
+      system 'rm -rf $tempdir';
   };
 }
 
+
+sub _unpackModulesRestoreData
+{
+  my ($self, $tempdir) = @_;
+
+  my $unpackCmd = "tar xzf $tempdir/eboxbackup/files.tgz -C $tempdir/eboxbackup";
+
+  system $unpackCmd;
+
+  if ($? != 0) {
+    system 'rm -rf $tempdir';
+    throw EBox::Exceptions::External(
+				     __('Could not unpack the backup')
+				    );
+  }
+}
+
+sub _restoreModule
+{
+  my ($self, $mod, $tempdir, $options_r) = @_;
+  my $modname = $mod->name();
+
+  # update progress indicator 
+  my $progress = $options_r->{progress};
+  $progress->notifyTick(); 
+  $progress->setMessage($modname);
+
+  if (not -e "$tempdir/eboxbackup/$modname.bak") {
+    EBox::error("Restore data not found for module $modname. Skipping $modname restore");
+    return 0;
+  }
+
+  EBox::debug("Restoring $modname from backup data");
+  $mod->setAsChanged(); # we set as changed first because it is not
+  # guaranteed that a failed backup will not
+  # change state
+  $mod->restoreBackup("$tempdir/eboxbackup", 
+		      fullRestore => $options_r->{fullRestore},
+		      dataRestore => $options_r->{dataRestore},
+		     );
+
+  $self->_migratePackage($mod->package());
+  
+  return 1;
+}
+
+
+sub _revokeRestore
+{
+  my ($self, $restored_r) = @_;
+
+  EBox::debug('revoking restore for all modules');
+  foreach my $restname (@{ $restored_r }) {
+    my $restmod = EBox::Global->modInstance($restname);
+    try {
+      $restmod->revokeConfig();
+      # XXX remember no-gconf changes are not revoked!
+      EBox::debug("Revoked changes in $restname module");
+    }
+    otherwise {
+      EBox::debug("$restname has not changes to be revoked" );
+    };
+  }
+}
+
 sub _migratePackage
 {
 	my ($self, $package) = @_;
 	my $migrationdir = EBox::Config::lib() . "/$package/migration";
 	
 	if (-d $migrationdir) {
-		my $migration = EBox::Config::libexec() . '/ebox-migrate';
+		my $migration = EBox::Config::pkgdata() . '/ebox-migrate';
 		try {
 			EBox::Sudo::command("$migration $migrationdir");
 		} catch EBox::Exceptions::Internal with {
@@ -884,15 +1073,63 @@ sub _migratePackage
 
 sub _modInstancesForRestore
 {
-  my ($self) = @_;
+  my ($self, $archive, %options) = @_;
+  my $forceDependencies = $options{forceDependencies};
+
+  my $anyModuleInBackup = any( @{ $self->_modulesInBackup($archive) } );
+
   my $global = EBox::Global->getInstance();
 
-  # we remove global module because it will be the last to be restored
-  my @modules   =  grep { $_->name ne 'global' } @{$global->modInstances() };   
+  my @modules =  @{ $global->modInstances() };
+
+
+  # if we have a module list we check it and only keep those modules
+  if (exists $options{modsToRestore}) {
+    my @modsToRestore =  @{ $options{modsToRestore} };
+    foreach my $m (@modsToRestore) {
+      if (not( $m eq $anyModuleInBackup)) {
+	throw EBox::Exceptions::External(
+					 __x(
+					     'No module {m} found in backup',
+					     'm' => $m
+					    )
+					);
+      }
+    }
+
+    # we use the module list instead of the full list of backup's module
+    $anyModuleInBackup = any @modsToRestore;
+  }
+
+
+  # we restore the intersection between the installed modules AND the modules in
+  # the backup archive
+  @modules = grep {
+    $_->name eq $anyModuleInBackup
+  } @modules;
+
+
+
+
+
+  # we remove global module because it will not  be restored
+  @modules   =  grep { $_->name ne 'global' } @modules;   
+  # XXX temporally skipping log module
+  @modules = grep {  $_->name ne 'logs' } @modules;
+
+  if (not @modules) {
+    throw EBox::Exceptions::External(
+	      __('No modules to restore')
+				    );
+  }
+
 
   # check modules dependencies
-  foreach my $mod (@modules) {
-    $self->_checkModDeps($mod->name);
+  if (not $forceDependencies) {
+    foreach my $mod (@modules) {
+      $self->_checkModDeps($mod->name);
+    }
+    
   }
 
   # we sort the modules list with a restore-safe order
@@ -910,13 +1147,20 @@ sub _modInstancesForRestore
     $aDependsB <=> $bDependsA;
   } @modules;
 
-  # we restore global in last place to avoid possible problems 
-  push @modules, $global; 
-
   return \@modules;
 }
 
 
+sub _modulesInBackup
+{
+  my ($self, $archive) = @_;
+
+  my $tempDir = $self->_unpackArchive($archive, 'modules'); 
+  my $modulesString = read_file("$tempDir/eboxbackup/modules"); 
+
+  my @modules = split '\s', $modulesString;
+  return \@modules;
+}
 
 
 sub _checkModDeps
@@ -937,7 +1181,7 @@ sub _checkModDeps
 
   if (not defined $mod) {
     if ($level == 0) {
-      throw EBox::Exceptions::Internal ("$topModule can not be created again");
+      throw EBox::Exceptions::Internal ("$topModule cannot be created again");
     }
     else {
       throw EBox::Exceptions::External __x('Unresolved restore dependency for module {topModule}: {modName} is not installed', topModule => $topModule, modName => $modName  );
@@ -954,52 +1198,6 @@ sub _checkModDeps
   }
 }
 
-#
-# Procedure: searchBackupFileInDiscs
-#
-#    searches the CD and DVD disks for a archive file
-#
-# Returns:
-#      the path to the first file found or undef if none is found
-#   	
-# 
-sub searchBackupFileInDiscs
-{
-  return EBox::Backup::OpticalDiscDrives::searchFileInDiscs($DISC_BACKUP_FILE);
-}
-
-#
-# Method: restoreBackupFromDisc
-#
-#      restores from a DVD or CD-ROM disk
-#
-# Limitation:
-#
-#   if we have multiples backups in differents disks or in the same disk, we don't know beforehand wich one will be used
-#
-# Parameters:
-#
-#       fullRestore  - wether do a full restore or  restore only configuration (default: false)
-#       
-
-sub restoreBackupFromDisc
-{
-  my ($self,  %options) = @_;
-  validate_with (params => [%options],
-		 spec => { fullRestore => { default => 0 },  });
-
-  my $discFileInfo = searchBackupFileInDiscs();
-  if (!defined $discFileInfo) {
-    throw EBox::Exceptions::External(__('Insert a backup disc and try again, please'));
-  }
-
-  try {
-    $self->restoreBackup($discFileInfo->{file}, %options);
-  }
-  finally {
-      EBox::Backup::OpticalDiscDrives::ejectDisc($discFileInfo->{device});
-  };
-}
 
 
 sub _checkId
@@ -1024,5 +1222,13 @@ sub _backupFileById
     return $file;
 }
 
+#  Function: _facilitiesForDiskUsage
+#  Overrides: EBox::Report::DiskUsageProvider::_faciltiesForDiskUsage
+sub _facilitiesForDiskUsage
+{
+  return {
+	  __(q{Backup's archives}) => [ backupDir() ],
+	 }
+}
 
 1;
