@@ -1,4 +1,4 @@
-# Copyright (C) 2005 Warp Networks S.L., DBS Servicios Informaticos S.L.
+# Copyright (C) 2007 Warp Networks S.L.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2, as
@@ -24,13 +24,21 @@ use EBox::Gettext;
 use EBox::Global;
 use EBox::Exceptions::NotImplemented;
 
+# Dependencies
+use Error qw(:try);
+use Clone;
+
 sub new # (cgi=?)
 {
 	my $class = shift;
 	my %params = @_;
-	my $tableModel = delete $params{'tableModel'};	
-	my $self = $class->SUPER::new('template' => '/ajax/tableBody.mas', 
-				@_);
+	my $tableModel = delete $params{'tableModel'};
+	my $template;
+	if (defined($tableModel)) {
+		$template = $tableModel->Viewer();
+	}
+	my $self = $class->SUPER::new('template' => $template,
+                                      @_);
 	$self->{'tableModel'} = $tableModel;
 	bless($self, $class);
 	return  $self;
@@ -39,44 +47,30 @@ sub new # (cgi=?)
 sub getParams
 {
 	my $self = shift;
-	
+
 	my $tableDesc = $self->{'tableModel'}->table()->{'tableDescription'};
 
 	my %params;
 	foreach my $field (@{$tableDesc}) {
-
-		my $type = $field->type();
-
 		foreach my $fieldName ($field->fields()) {
-			my $value = $self->param($fieldName);
-		
-			if ($field->{'optional'} == 1 and $type  ne 'boolean') {
-				if (not defined($value)) {
-					$value = "";
-				}
-				
-			} else {
-
-				if ($type ne 'boolean') {
-					$self->_requireParam($fieldName,
-						$field->printableName());
-				} else {
-					if ($value) {
-						$value = 1;
-					} else {
-						$value = 0;
-					}
-				}
-			}
-
-			$params{$fieldName} = $value;
+                    my $value;
+                    if ( $field->allowUnsafeChars() ) {
+                        $value = $self->unsafeParam($fieldName);
+                    } else {
+			$value = $self->param($fieldName);
+                    }
+            # TODO Review code to see if we are actually checking
+            # types which are not optional
+                    $params{$fieldName} = $value;
 		}
 	}
 
 	$params{'id'} = $self->param('id');
+	$params{'filter'} = $self->param('filter');
 
 	return %params;
 }
+
 sub addRow
 {
 	my $self = shift;
@@ -114,8 +108,9 @@ sub removeRow()
 	
 	$self->_requireParam('id');
 	my $id = $self->param('id');
+	my $force = $self->param('force');
 
-	$model->removeRow($id);
+	$model->removeRow($id, $force);
 	
 }
 
@@ -125,8 +120,9 @@ sub editField
 
 	my $model = $self->{'tableModel'};
 	my %params = $self->getParams();
-	$model->setRow($self->getParams());
-	
+	my $force = $self->param('force');
+	$model->setRow($force, %params);
+
 	my $editField = $self->param('editfield');
 	if (not $editField) {
 		return;
@@ -144,9 +140,10 @@ sub editField
 			}
 	}
 
-
 }
 
+
+# Method to refresh the table by calling rows method
 sub refreshTable
 {
 	my $self = shift;
@@ -154,18 +151,31 @@ sub refreshTable
 	my $model = $self->{'tableModel'};
 	my $global = EBox::Global->getInstance(); 
 
+	my $filter = $self->param('filter');
+	my $page = $self->param('page');
+	my $pageSize = $self->param('pageSize');
+        if ( defined ( $pageSize )) {
+            $model->setPageSize($pageSize);
+        }
+	my $rows = $model->rows($filter, $page);
+	my $tpages = $model->pages($filter);
 	my @params;
-	push(@params, 'data' => $model->rows() );
-	push(@params, 'dataTable' => $model->tableInfo());
+	push(@params, 'data' => $rows);
+	push(@params, 'dataTable' => $model->table());
+	push(@params, 'model' => $model);
 	push(@params, 'action' => $self->{'action'});
 	push(@params, 'editid' => $self->param('editid'));
 	push(@params, 'hasChanged' => $global->unsaved());
-	
+	push(@params, 'filter' => $filter);
+	push(@params, 'page' => $page);
+	push(@params, 'tpages' => $tpages);
 
 	$self->{'params'} = \@params;
 
 }
 
+# Group: Protected methods
+
 sub _process
 {
 	my $self = shift;
@@ -174,6 +184,14 @@ sub _process
 	my $action = $self->param('action');
 	$self->{'action'} = $action;
 
+	my $model = $self->{'tableModel'};
+	
+	my $directory = $self->param('directory');
+	if ($directory) {
+
+		$model->setDirectory($directory);
+	}
+
 	if ($action eq 'edit') {
 
 		$self->editField();
@@ -206,7 +224,13 @@ sub _process
 	
 		$self->refreshTable();
 
-	}
+        } elsif ($action eq 'view') {
+                # This action will show the whole table (including the
+                # table header similarly View Base CGI but inheriting
+                # from ClientRawBase instead of ClientBase
+                $self->{template} = $model->Viewer();
+                $self->refreshTable();
+        }
 }
 
 sub _print
