@@ -27,7 +27,10 @@ use EBox::Exceptions::Internal;
 use EBox::Gettext;
 use EBox::GConfState;
 use EBox::GConfConfig;
+
+# Core modules
 use File::Basename;
+use File::Copy::Recursive;
 
 sub _create # (name)
 {
@@ -68,6 +71,7 @@ sub _helper
 sub _config
 {
 	my $self = shift;
+	$self->{gconf}->clear_cache();
 	$self->{helper} = $self->{config};
 }
 
@@ -78,14 +82,20 @@ sub _state
 }
 
 
+sub initChangedState
+ {
+   my ($self) = @_;
 
-sub saveAsBaseline
-{
-  my ($self) = @_;
-  $self->_backup();
+   my $global = EBox::Global->getInstance();
+   $global->modIsChanged($self->name) and 
+     throw EBox::Exceptions::Internal($self->name . ' module already has changed state');
+
+
+   $self->_dump_to_file();
 }
 
 
+
 # we override aroundRestoreconfig to save gconf data before dump module config
 sub aroundRestoreConfig
 {
@@ -101,29 +111,40 @@ sub _load_from_file # (dir?, key?)
   my ($self, $dir, $key) = @_;
   ($dir) or $dir = EBox::Config::conf;
 
+  $self->_config();
+
   my $file =  $self->_bak_file_from_dir($dir);
-  -f $file or throw EBox::Exceptions::Internal("Backup file missing: ".
-					       "$file.");
+  if (not  -f $file)  {
+    EBox::error("GConf backup file missing for module " . $self->name);
+    return;
+  }
+
   ($key) or $key = $self->_key("");
   $self->_delete_dir_internal($key);
-  `/usr/bin/gconftool --load=$file $key` and
+
+  system "/usr/bin/gconftool --load=$file $key";
+  if (not ($? == 0)) {
     throw EBox::Exceptions::Internal("Error while restoring " .
 				     "configuration from $file");
+  }
+
 }
 
 
 # we override aroundDumpConfig to save gconf data before dump module config
 sub aroundDumpConfig
 {
-  my ($self, $dir) = @_;
+  my ($self, $dir, @options) = @_;
   $self->_dump_to_file($dir);  
-  $self->dumpConfig($dir);     
+  $self->dumpConfig($dir, @options);     
 }
 
 # dumps GConf entries to a file in the dir specified
 sub _dump_to_file # (dir?) 
 {
 	my ($self, $dir) = @_;
+	$self->_config();
+
 	my $key = "/ebox/modules/" . $self->name;
 	($dir) or $dir = EBox::Config::conf;
 	my $file = $self->_bak_file_from_dir($dir);
@@ -164,6 +185,8 @@ sub revokeConfig
 	my $ro = $self->{ro};
 	$self->{ro} = undef;
 	$self->_load_from_file();
+        # Restore <EBox::Types::File> which content a file
+        $self->_restoreFilesFromBackup();
 	$self->{ro} = $ro;
 }
 
@@ -184,6 +207,8 @@ sub _saveConfig
 			 . " module: " . $self->name() . "\n");
 	}
 	$self->_dump_to_file();
+        # Backup a copy of the content of <EBox::Types::File>
+        $self->_backupFiles();
 	$self->_load_from_file(undef, "/ebox-ro/modules/". $self->name());
 }
 
@@ -235,7 +260,12 @@ sub _gconf_wrapper # (method, @params?)
 		if ($ret){
 			@array = &$code($self->gconf, @parms);
 		} else {
-			$scalar = &$code($self->gconf, @parms);
+            {
+                # Silent really weird warning which is likeley due to
+                # the perl version
+                no warnings;
+			    $scalar = &$code($self->gconf, @parms);
+            }
 		}	
 	};
 	if ($@) {
@@ -396,7 +426,9 @@ sub _all_entries # (key)
 {
 	my ($self, $key) = @_;
 	$key = $self->_key($key);
-	return $self->_gconf_wrapper("all_entries", $key);
+	my @entries = $self->_gconf_wrapper("all_entries", $key);
+	my @entrypaths = map { $_->{key} } @entries;
+	return @entrypaths;
 }
 
 #
@@ -411,7 +443,7 @@ sub _all_entries # (key)
 #
 # Returns:
 #
-#       A ref to an array of strings - Each string contains an entry
+#       array of strings - Each string contains an entry
 #
 #
 sub all_entries # (key) 
@@ -895,7 +927,7 @@ sub _delete_dir_internal # (key)
 	my ($self, $dir) = @_;
 	my @keys = $self->_gconf_wrapper("all_entries", $dir);
 	foreach (@keys) {
-		$self->_gconf_wrapper("unset", $_);
+		$self->_gconf_wrapper("unset", $_->{key});
 	}
 	@keys = $self->_gconf_wrapper("all_dirs", $dir);
 	foreach (@keys) {
@@ -926,16 +958,130 @@ sub _delete_dir_internal # (key)
 sub get_unique_id # (prefix, directory?)
 {
 	my ($self, $prefix, $directory) = @_;
+	return $self->_get_unique_id($prefix, $directory, 'dir_exists');
+}
+
+#
+# Method: st_get_unique_id 
+#
+# 	It generates a unique random identifier with a leading
+#       prefix in the root of the module's state
+#       namespace, if directory is passed, it will
+#       be added to the path. Note that it does not create the entry, it
+#       just returns a unique identifier, so it is up to you to create the
+#       proper entry
+#
+# Parameters:
+#
+#       prefix  - prefix to be added to the root of the module's state namespace
+#	directory - if the directory is passed, this is added to the path
+#
+# Returns:
+#
+#	string - unique identifier
+sub st_get_unique_id # (prefix, directory?)
+{
+	my ($self, $prefix, $directory) = @_;
+	return $self->_get_unique_id($prefix, $directory, 'st_dir_exists');
+}
+
+
+sub _get_unique_id
+{
+         my ($self, $prefix, $directory, $dirExistsMethod) = @_;
+
 	if ($directory) {
 		$directory .= '/';
 	} else {
 		$directory = "";
 	}
 	my $id = $prefix . int(rand(10000));
-	while ($self->dir_exists($directory . $id)) {
+	while ($self->$dirExistsMethod($directory . $id)) {
 		$id = $prefix . int(rand(10000));
 	}
 	return $id;
 }
 
+# Method: _restoreFilesFromBackup
+#
+#     Restore the files stored using <EBox::Types::File> type by
+#     models, therefore the model itself must be a
+#     <EBox::Model::ModelProvider> instance
+#
+sub _restoreFilesFromBackup
+{
+
+    my ($self) = @_;
+
+    my $filePaths = $self->_filePaths();
+    if ( @{$filePaths} > 0 ) {
+        foreach my $filePath (@{$filePaths}) {
+            if ( -f $filePath . '.bak' ) {
+                File::Copy::Recursive::fcopy($filePath . '.bak', $filePath)
+                    or throw EBox::Exceptions::Internal('Cannot copy from ' .
+                                                        $filePath . ".bak to $filePath : $!");
+            }
+        }
+    }
+}
+
+# Method: _backupFiles
+#
+#     Backup the files stored using <EBox::Types::File> type by
+#     models, therefore the model itself must be a
+#     <EBox::Model::ModelProvider> instance
+#
+sub _backupFiles
+{
+
+    my ($self) = @_;
+
+    my $filePaths = $self->_filePaths();
+    if ( @{$filePaths} > 0 ) {
+        foreach my $filePath (@{$filePaths}) {
+            if ( -f $filePath ) {
+                File::Copy::Recursive::fcopy($filePath, $filePath  . '.bak')
+                    or throw EBox::Exceptions::Internal('Cannot copy from '
+                                                        . "$filePath to $filePath"
+                                                        . ".bak: $!");
+            }
+        }
+    }
+}
+
+
+# Method to get those file full paths which are enclosed within this
+# model provider
+# Return value: array ref with the current file paths
+sub _filePaths
+{
+    my ($self) = @_;
+
+    unless ( $self->isa('EBox::Model::ModelProvider') ) {
+        return [];
+    }
+
+    my @filePaths = ();
+    my $models = $self->models();
+    foreach my $model (@{$models}) {
+        my $hasFileType = 0;
+        foreach my $fieldName ( @{$model->fields()} ) {
+            if ( $model->fieldHeader($fieldName)->isa('EBox::Types::File')) {
+                $hasFileType = 1;
+                last;
+            }
+        }
+        if ( $hasFileType ) {
+            foreach my $modelRow (@{$model->rows()}) {
+                foreach my $field (@{$modelRow->{values}} ) {
+                    if ( $field->isa('EBox::Types::File' )) {
+                        push ( @filePaths, $field->path()) if ( $field->path() ne '');
+                    }
+                }
+            }
+        }
+    }
+    return \@filePaths;
+}
+
 1;
