@@ -28,20 +28,25 @@ use EBox::Exceptions::Internal;
 use EBox::Exceptions::Lock;
 use EBox::Gettext;
 use EBox::FileSystem;
+use EBox::ServiceModule::Manager;
 use HTML::Mason;
 use File::Temp qw(tempfile);
 use Fcntl qw(:flock);
 use Error qw(:try);
 use Params::Validate qw(validate_pos validate_with SCALAR HASHREF ARRAYREF);
 
+use constant SER_IFACE_CLASS => 'EBox::ServiceModule::ServiceInterface';
+
 # Method: _create 
 #
 #   	Base constructor for a module
 #
 # Parameters:
 #
-#       name - module's name
-#	domain - locale domain
+#       name - String module's name
+#	domain - String locale domain
+#       printableName - String printable module's name
+#       title - String the module's title
 #
 # Returns:
 #
@@ -58,6 +63,7 @@ sub _create # (name, domain?)
 	$self->{name} = delete $opts{name};
 	$self->{domain} = delete $opts{domain};
 	$self->{title} = delete $opts{title};
+        $self->{printableName} = delete $opts{printableName};
 	unless (defined($self->{name})) {
 		use Devel::StackTrace;
 		my $trace = Devel::StackTrace->new;
@@ -104,8 +110,9 @@ sub _regenConfig
 
 # Method: _restartService
 #
-#	This method will try to restart the module's service by means of calling
-#	_regenConfig
+#        This method will try to restart the module's service by means of
+#        calling _regenConfig. The method call has the named
+#        parameter restart with true value
 #
 sub restartService
 {
@@ -114,6 +121,12 @@ sub restartService
 	$self->_lock();
 	my $global = EBox::Global->getInstance();
 	my $log = EBox::logger;
+	
+	if ($self->isa(SER_IFACE_CLASS) and not $self->isEnabled()) {
+		$log->info("Skipping restart for $self->{name} as it's disabled");
+		return;
+	}
+
 	$log->info("Restarting service for module: " . $self->name);
 	try {
 		$self->_regenConfig('restart' => 1);
@@ -153,13 +166,47 @@ sub saveConfig
 	my $self = shift;
 
 	$self->_lock();
-	my $global = EBox::Global->getInstance();
-	my $log = EBox::logger;
-	$log->info("Restarting service for module: " . $self->name);
-	$self->_saveConfig();
-	$self->_unlock();
+	try {
+	  my $global = EBox::Global->getInstance();
+	  my $log = EBox::logger;
+	  $log->info("Restarting service for module: " . $self->name);
+	  $self->_saveConfig();
+	}
+	finally {
+	  $self->_unlock();
+	};
 }
 
+# Method: saveConfigRecursive
+#
+#   Save module config and the modules which depends on recursively
+#
+sub saveConfigRecursive
+{
+    my ($self) = @_;
+
+    $self->_saveConfigRecursive($self->name);
+}
+
+# Method: _saveConfigRecursive
+#
+#   Save module config and the modules which depends on recursively
+#
+sub _saveConfigRecursive
+{
+    my ($self, $module) = @_;
+
+    my $global = EBox::Global->getInstance();
+    for my $dependency (@{$global->modDepends($module)}) {
+        $self->_saveConfigRecursive($dependency); 
+    }
+
+    my $modInstance = EBox::Global->modInstance($module);
+    $modInstance->saveConfig();
+}
+
+
+
 sub _unlock
 {
 	my $self = shift;
@@ -234,27 +281,36 @@ sub setAsChanged
 #  dir - directory used for the backup operation
 #  (named parameters following)
 #  fullBackup - wether we want to do a full restore as opposed a configuration-only restore (default: false)
-#  directlyToDisc - wether the backup is written directly to a CD
-#
+# bug - wether we are making a bug report instead of a normal backup 
 sub makeBackup # (dir, %options) 
 {
   my ($self, $dir, %options) = @_;
   defined $dir or throw EBox::Exceptions::InvalidArgument('directory');
   validate_with ( params => [%options],
-		  spec =>  { fullBackup     => { default => 0},  
-			     directlyToDisc => { default => 0},
+		  spec =>  { 
+			    fullBackup     => { default => 0},  
+                             bug            => { default => 0 },
+			     progress        => {optional => 1},
 			   } );
 
   my $backupDir = $self->_createBackupDir($dir);
 
-  $self->aroundDumpConfig($backupDir);
+  $self->aroundDumpConfig($backupDir, %options);
 
-  if ($self->can('extendedBackup') and $options{fullBackup}) {
-    $self->_bootstrapExtendedBackup($backupDir, %options);
+  if ($options{fullBackup}) {
+    $self->extendedBackup(dir => $backupDir, %options);
   }
 
 }
 
+
+# default implementation: do nothing
+sub extendedBackup
+{
+  # my %params = @_;
+  # my $dir = $params{dir};
+}
+
 # Method: backupDir
 #
 # Parameters:
@@ -301,30 +357,8 @@ sub _createBackupDir
   return $backupDir;
 }
 
-sub _bootstrapExtendedBackup
-{
-  my ($self, $dir, %options) = @_;
-
-  # save version
-  if ($self->can('version')) {
-    $self->_dump_version($dir);
-  }
-
-  $self->extendedBackup(dir => $dir, %options);
-}
 
 
-sub _dump_version
-{
-  my ($self, $dir) = @_;
-
-  my $file = "$dir/version";
-  my $versionInfo = $self->version();
-
-  open my $FH, ">$file" or throw EBox::Exceptions::Internal('Cannot create version backup file');
-  print $FH $versionInfo;
-  close $FH;
-}
 
 #
 # Method: restoreBackup
@@ -341,40 +375,42 @@ sub restoreBackup # (dir, %options)
   my ($self, $dir, %options) = @_;
   defined $dir or throw EBox::Exceptions::InvalidArgument('directory');
   validate_with ( params => [%options],
-		  spec =>  { fullRestore => { default => 0}   } );
+		  spec =>  { 
+			    fullRestore => { default => 0},
+			    dataRestore => {
+					    default => 0,
+					    # incompatible with fullRestore ..
+					    callbacks => {
+							  incompatibleRestores =>  
+							  sub {
+							    (not $_[0]) or (not $_[1]->{fullRestore})
+							  },
+							 },
+					   },
+			   },
+		);
   
   my $backupDir = $self->backupDir($dir);
   (-d $backupDir) or throw EBox::Exceptions::Internal("$backupDir must be a directory");
 
-  $self->aroundRestoreConfig($backupDir);
+  if (not $options{dataRestore}) {
+    $self->aroundRestoreConfig($backupDir);
+  }
 
-  if ($options{fullRestore} and $self->can('extendedRestore')) {
-    $self->_bootstrapExtendedRestore($backupDir, %options);
+
+  if ($options{fullRestore} or $options{dataRestore}) {
+    $self->extendedRestore(dir => $backupDir, %options);
   }
 }
 
-sub _bootstrapExtendedRestore
-{
-  my ($self, $dir, @options) = @_;
 
-  my $version = $self->_read_version($dir);
-  $self->extendedRestore(dir => $dir, version => $version, @options);
+# default implementation: do nothing
+sub extendedRestore
+{
+  # my %params = @_;
+  # my $dir = $params{dir};
 }
 
-sub _read_version
-{
-  my ($self, $dir) = @_;
-  my $file = "$dir/version";
-
-  return undef if (! -f $file);
-  
-  open my $FH, "<$file" or throw EBox::Exceptions::Internal("Version info file cannot be opened");
-  my @versionInfo = <$FH>;
-  close $FH;
-
-  my $versionInfo = join "\n", @versionInfo;
-  return $versionInfo
-}
 
 
 sub _bak_file_from_dir
@@ -413,8 +449,8 @@ sub restoreDependencies
 #
 sub dumpConfig
 {
-  my ($self, $dir) = @_;
-  validate_pos(@_, 1, 1);
+  my ($self, $dir, %options) = @_;
+
 }
 
 
@@ -434,10 +470,10 @@ sub dumpConfig
 #
 sub aroundDumpConfig
 {
-  my ($self, $dir) = @_;
-  validate_pos(@_, 1, 1);
+  my ($self, $dir, @options) = @_;
 
-  $self->dumpConfig($dir);
+
+  $self->dumpConfig($dir, @options);
 }
 
 
@@ -479,24 +515,29 @@ sub aroundRestoreConfig
 }
 
 
+
+# Method: initChangedState
 #
-# Method: saveAsBaseline
-#
-#	Set the actual state as baseline configuration of the module. This base
-#	implementation is empty
-#
-sub saveAsBaseline
+#    called before module is in the changed state. 
+sub initChangedState
 {
+  my ($self) = @_;
+
+   my $global = EBox::Global->getInstance();
+   $global->modIsChanged($self->name) and 
+     throw EBox::Exceptions::Internal($self->name . ' module already has changed state');
+
+
 }
 
 #
-# Method: name 
+# Method: name
 #
-#	Returns the module name of the current module instance   
+#	Return the module name of the current module instance
 #
 # Returns:
 #
-#      	strings - name 
+#      	strings - name
 #
 sub name
 {
@@ -505,9 +546,9 @@ sub name
 }
 
 #
-# Method: setName 
+# Method: setName
 #
-#	Sets the module name for the current module instance
+#	Set the module name for the current module instance
 #
 # Parameters:
 #
@@ -520,6 +561,47 @@ sub setName # (name)
 	$self->{name} = $name;
 }
 
+# Method: printableName
+#
+#       Return the printable module name of the current module
+#       instance
+#
+# Returns:
+#
+#       String - the printable name
+#
+sub printableName
+{
+
+    my ($self) = @_;
+
+    if ( $self->{printableName} ) {
+        return $self->{printableName};
+    } else {
+        return $self->name();
+    }
+
+}
+
+# Method: setPrintableName
+#
+#       Set the printable module name of the current module
+#       instance
+#
+# Parameters:
+#
+#       printableName - String the printable name
+#
+sub setPrintableName
+{
+
+    my ($self, $printableName) = @_;
+
+    $self->{printableName} = $printableName;
+
+}
+
+
 #
 # Method: title
 #
@@ -585,12 +667,20 @@ sub menu
 }
 
 #
-# Method: summary 
+# Method: summary
 #
-#	This method returns the summary for the module. What it returns it will 
-#	be added up to the common summry page. It should be overriden by 
+#	Return the summary for the module. What it returns will
+#	be added up to the common summary page. It should be overriden by 
 #	subclasses as needed
 #
+# Returns:
+#
+#       <EBox::Summary::Module> - the summary for the module
+#
+#       <EBox::Summary::Composite> - if several modules are included
+#       in the same summary it may return an instance of this class
+#       which consists of <EBox::Summary::Module> as elements
+#
 sub summary
 {
 	# default empty implementation
@@ -598,11 +688,15 @@ sub summary
 }
 
 #
-# Method: statusSummary 
+# Method: statusSummary
 #
-#	This method returns the summary for the module. What it returns it will 
-#	be added up to the common summry page. It should be overriden by 
-#	subclasses as needed
+#	Return the status summary for the module. What it returns will
+#	be added up to the common status summary page. It should be overriden by
+#	subclasses as needed.
+#
+# Returns:
+#
+#       <EBox::Summary::Status> - the summary status for the module
 #
 sub statusSummary
 {
@@ -721,6 +815,16 @@ sub writeConfFile # (file, component, params, defaults)
 	my ($self, $file, $compname, $params, $defaults) = @_;
 	validate_pos(@_, 1, { type =>  SCALAR }, { type => SCALAR }, { type => ARRAYREF, default => [] }, { type => HASHREF, optional => 1 });
 
+    my $manager;
+    if ($self->isa('EBox::ServiceModule::ServiceInterface')) {
+        $manager = new EBox::ServiceModule::Manager();
+        if ($manager->skipModification($self->serviceModuleName(), $file)) {
+            EBox::info("Skiping modification of $file");
+            return;
+        }
+    }
+
+
 	my ($fh,$tmpfile) = tempfile(DIR => EBox::Config::tmp);
 	unless($fh) {
 		throw EBox::Exceptions::Internal(
@@ -751,8 +855,12 @@ sub writeConfFile # (file, component, params, defaults)
 	EBox::Sudo::root("/bin/mv $tmpfile  $file");
 	EBox::Sudo::root("/bin/chmod $mode $file");
 	EBox::Sudo::root("/bin/chown $uid.$gid $file");
+
+    if ($self->isa('EBox::ServiceModule::ServiceInterface')) {
+        $manager->updateFileDigest($self->serviceModuleName(), $file);
+    }
+
+
 }
 
-
-
 1;
