@@ -37,18 +37,18 @@ use EBox::Sudo qw( :all );
 sub manage # (daemon,action)
 {
 	my ($daemon, $action) = @_;
-	(-d "/var/service/$daemon") or
+	(-f "/etc/event.d/$daemon") or
 		throw EBox::Exceptions::Internal("No such daemon: $daemon");
 
 	if ( $action eq 'start' ) {
-		root("/usr/bin/runsvctrl up /var/service/$daemon");
+		root("start $daemon");
 	}
 	elsif ( $action eq 'stop' ) {
-		root("/usr/bin/runsvctrl down /var/service/$daemon");
+		root("stop $daemon");
 	}
 	elsif ( $action eq 'restart') {
-		root("/usr/bin/runsvctrl down /var/service/$daemon");
-		root("/usr/bin/runsvctrl up /var/service/$daemon");
+		root("stop $daemon");
+		root("start $daemon");
 	}
 	else {
 		throw EBox::Exceptions::Internal("Bad argument: $action");
@@ -66,19 +66,19 @@ sub manage # (daemon,action)
 #
 #   Exceptions:
 #
-#       Internal - Bad argument
+#       <EBox::Exceptions::Internal> - Bad argument
 #
 sub running # (daemon)
 {
 	my ($daemon) = @_;
-	(-d "/var/service/$daemon") or
+	(-f "/etc/event.d/$daemon") or
 		throw EBox::Exceptions::Internal("No such daemon: $daemon");
 
-	my $output = root("/usr/bin/runsvstat /var/service/$daemon");
+	my $output = root("status $daemon");
 	my $status = @{$output}[0];
-	if ($status =~ m{^/var/service/$daemon: run}) {
+	if ($status =~ m{^$daemon .* running}) {
 		return 1;
-	} elsif ($status =~ m{^/var/service/$daemon: down}) {
+	} elsif ($status =~ m{^$daemon .*stop}) {
 		return undef;
 	} else {
 		throw EBox::Exceptions::Internal("Error getting status: $daemon");
